from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields, ValidationError
import re
from dataclasses import dataclass, field

regex = re.compile(r"^(\d+)-(\d+)$")


def validate_str(string):
    reg = re.match(regex, string)
    if not reg:
        raise ValidationError("Incorrect format, use: (integer)-(integer)")
    if int(reg.group(1)) > int(reg.group(2)):
        raise ValidationError("Range has to be in ascending order")
    return reg


def validate_list(_list):
    if all(isinstance(e, int) for e in _list):
        return True
    raise ValidationError("Only Integers allowed in the list")


class FaradayRangeField(fields.Field):
    def _serialize(self, value, attr, obj, **kwargs):
        if value is None:
            return ""
        if isinstance(value, str):
            return validate_str(value).group(0)
        if isinstance(value, list):
            validate_list(value)
            return f"{value[0]}-{value[-1]}"
        raise ValidationError("Invalid Data Type")

    def _deserialize(self, value, attr, data, **kwargs):
        if value is None:
            return []
        if isinstance(value, list):
            validate_list(value)
            return value
        if isinstance(value, str):
            reg = validate_str(value)
            return list(range(int(reg.group(1)), int(reg.group(2)) + 1))
        raise ValidationError("Invalid Data Type")


@dataclass
class FaradayRange:
    data: list = field(default_factory=list)
    class_name: str = field(default="range", init=False)
    base: str = field(default="string", init=False)


class FaradayRangeSchema(TypeSchema):
    data = FaradayRangeField()
    type = FaradayRange



from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields
from dataclasses import dataclass, field


@dataclass
class FaradayBoolean:
    data: bool = bool()
    class_name: str = field(default="boolean", init=False)
    base: str = field(default="boolean", init=False)


class FaradayBooleanSchema(TypeSchema):
    data = fields.Boolean()
    type = FaradayBoolean



from typing import Any
from marshmallow import Schema, post_load


class TypeSchema(Schema):

    # Overridden by data class values
    data: Any
    type: Any

    @post_load
    def create_instance(self, data, **kwargs):
        return self.type(**data)

    def to_obj(self):
        return self.type().class_name



"""Top-level package for faraday_agent_parameters_types."""

__author__ = """Faraday Development Team"""
__email__ = "devel@infobytesec.com"
__version__ = "1.0.0"

from pathlib import Path
from typing import Union


def manifests_folder() -> Union[Path, str]:

    return Path(__file__).parent / "static" / "manifests"



import json
from functools import lru_cache

from faraday_agent_parameters_types.data_types import DATA_TYPE
from typing import Union, List, Any
from marshmallow import ValidationError
from faraday_agent_parameters_types.faraday_agent_parameters_types import TypeSchema
from pathlib import Path
from packaging.version import parse, Version

manifests_folder = Path(__file__).parent / "static" / "manifests"


def get_schema(type_schema: Union[str, TypeSchema]) -> TypeSchema:
    if isinstance(type_schema, TypeSchema):
        return type_schema
    if isinstance(type_schema, str):
        if type_schema in DATA_TYPE:
            return DATA_TYPE[type_schema]
    raise ValidationError("Invalid Data Type")


def type_validate(type_schema: Union[str, TypeSchema, List[Union[str, TypeSchema]]], data) -> dict:
    if isinstance(type_schema, list):
        errors = {}
        for t in type_schema:
            error = get_schema(t).validate({"data": data})
            if not error:
                return {}
            else:
                errors[t] = error
    else:
        errors = get_schema(type_schema).validate({"data": data})
    return errors


def deserialize_param(type_schema: Union[str, TypeSchema, List[Union[str, TypeSchema]]], data, get_obj=False) -> Any:
    if isinstance(type_schema, list):
        for t in type_schema:
            error = get_schema(t).validate({"data": data})
            if not error:
                type_schema = t
                break
        else:
            raise ValidationError("Could not validate with any of the possible types")
    obj = get_schema(type_schema).load({"data": data})
    return obj if get_obj else obj.data


def serialize_param(type_schema: Union[str, TypeSchema, List[Union[str, TypeSchema]]], data, get_dict=False) -> Any:
    if isinstance(type_schema, list):
        for t in type_schema:
            error = get_schema(t).validate({"data": data})
            if not error:
                type_schema = t
                break
        else:
            raise ValidationError("Could not validate with any of the possible types")
    r_dict = get_schema(type_schema).dump({"data": data})
    return r_dict if get_dict else r_dict.get("data")


@lru_cache()
def get_manifests(version_requested: str = None) -> dict:
    all_manifests_dict = {}

    if version_requested is not None:
        version_requested = parse(version_requested)
        if not isinstance(version_requested, Version):
            raise ValueError("Version requested not valid")

    for path in manifests_folder.iterdir():
        if path.is_file():
            with path.open() as file:
                loaded_json = json.load(file)
                parsed_ver = parse(loaded_json["manifest_version"])
                if version_requested is not None and parsed_ver > version_requested:
                    continue
                manifest_name = loaded_json["name"]
                if manifest_name not in all_manifests_dict:
                    all_manifests_dict[manifest_name] = {}
                all_manifests_dict[manifest_name][parsed_ver] = loaded_json

    manifests_dict = {}
    for tool_name, tool in all_manifests_dict.items():
        manifests_dict[tool_name] = tool[max(tool.keys())]

    return manifests_dict



from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields
from dataclasses import dataclass, field


@dataclass
class FaradayUrl:
    data: str = str()
    class_name: str = field(default="url", init=False)
    base: str = field(default="string", init=False)


class FaradayUrlSchema(TypeSchema):
    data = fields.Url()
    type = FaradayUrl



from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields, ValidationError, validates
from dataclasses import dataclass, field


@dataclass
class FaradayList:
    data: list = field(default_factory=list)
    class_name: str = field(default="list", init=False)
    base: str = field(default="list", init=False)


class FaradayListSchema(TypeSchema):
    data = fields.List(fields.Raw())
    type = FaradayList
    _composed_list = []

    @validates("data")
    def validate_length_characters(self, items):
        for item in items:
            if not isinstance(item, self._composed_list):
                raise ValidationError("Item not valid in list")



from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields, ValidationError, validates
from dataclasses import dataclass, field


@dataclass
class FaradayString:
    data: str = str()
    class_name: str = field(default="string", init=False)
    base: str = field(default="string", init=False)


class FaradayStringSchema(TypeSchema):
    data = fields.String()
    type = FaradayString

    @validates("data")
    def validate_length_characters(self, text):
        if len(text) > 256:
            raise ValidationError(f"Max length 256. Your text {len(text)}")



from pathlib import Path
from unittest import mock

import pytest

from faraday_agent_parameters_types.utils import get_manifests

needed_manifest_fields = (
    "arguments",
    "environment_variables",
    "check_cmds",
    "cmd",
    "category",
    "name",
    "title",
    "website",
    "description",
    "image",
    "manifest_version",
    "repo_executor",
)

nullable_fields = ("website", "image")


def test_manifests_valid():
    manifests = get_manifests()
    assert manifests
    for manifest in manifests.values():
        # Check if field exists
        assert all(field in manifest for field in needed_manifest_fields)
        # Check if none
        for key, value in manifest.items():
            assert key in nullable_fields or value is not None


@mock.patch("faraday_agent_parameters_types.utils.manifests_folder", Path(__file__).parent / "test_manifests")
class Test_manifests_versions:
    def test_ask_for_low_nonexistant_version(self):
        manifests = get_manifests("0.1.0")
        assert not manifests

    def test_ask_for_low_unique_version(self):
        manifests = get_manifests("1.5.0")
        assert len(manifests) == 1
        for tool in manifests.values():
            assert tool["manifest_version"] == "1.5.0"

    def test_dont_ask_for_version(self):
        manifests = get_manifests()
        for name, tool in manifests.items():
            if name == "test":
                assert tool["manifest_version"] == "1.8.0"
            if name == "test2":
                assert tool["manifest_version"] == "1.6.0"

    def test_ask_for_low_version(self):
        manifests = get_manifests("1.7.0")
        for name, tool in manifests.items():
            if name == "test":
                assert tool["manifest_version"] == "1.7.0"
            if name == "test2":
                assert tool["manifest_version"] == "1.6.0"

    def test_incorrect_version_requested(self):
        with pytest.raises(ValueError) as error:
            get_manifests("hola")
        assert "Version requested not valid" in str(error.value)



from typing import Union
from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields
from dataclasses import dataclass, field
from ipaddress import IPv4Address, IPv6Address


@dataclass
class FaradayIP:
    data: Union[IPv4Address, IPv6Address] = IPv4Address("127.0.0.1")
    class_name: str = field(default="ip", init=False)
    base: str = field(default="string", init=False)


class FaradayIPSchema(TypeSchema):
    data = fields.IP()
    type = FaradayIP



from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields
from dataclasses import dataclass, field


@dataclass
class FaradayFloat:
    data: float = float()
    class_name: str = field(default="float", init=False)
    base: str = field(default="decimal", init=False)


class FaradayFloatSchema(TypeSchema):
    data = fields.Float()
    type = FaradayFloat



from faraday_agent_parameters_types.custom_types import (
    faraday_integer,
    faraday_string,
    faraday_boolean,
    faraday_list,
    faraday_int_range,
    faraday_ip,
    faraday_float,
    faraday_url,
)

DATA_TYPE = {
    "integer": faraday_integer.FaradayIntegerSchema(),
    "string": faraday_string.FaradayStringSchema(),
    "boolean": faraday_boolean.FaradayBooleanSchema(),
    "list": faraday_list.FaradayListSchema(),
    "range": faraday_int_range.FaradayRangeSchema(),
    "ip": faraday_ip.FaradayIPSchema(),
    "float": faraday_float.FaradayFloatSchema(),
    "url": faraday_url.FaradayUrlSchema(),
}



#!/usr/bin/env python

"""Tests for `faraday_agent_parameters_types` package."""

import pytest
from marshmallow import ValidationError, Schema

from faraday_agent_parameters_types.data_types import DATA_TYPE
from faraday_agent_parameters_types.utils import deserialize_param, serialize_param, type_validate

from faraday_agent_parameters_types.custom_types import (
    faraday_integer,
    faraday_string,
    faraday_boolean,
    faraday_list,
    faraday_int_range,
    faraday_ip,
    faraday_float,
    faraday_url,
)

from ipaddress import IPv4Address, IPv6Address

indentify_dict = [
    {"obj": {"type": "integer"}, "class": faraday_integer.FaradayIntegerSchema()},
    {"obj": {"type": "range"}, "class": faraday_int_range.FaradayRangeSchema()},
    {"obj": {"type": "string"}, "class": faraday_string.FaradayStringSchema()},
    {"obj": {"type": "boolean"}, "class": faraday_boolean.FaradayBooleanSchema()},
    {
        "obj": {"type": "list", "composed": ["integer", "string"]},
        "class": faraday_list.FaradayListSchema(),
    },
    {"obj": {"type": "ip"}, "class": faraday_ip.FaradayIPSchema()},
    {"obj": {"type": "float"}, "class": faraday_float.FaradayFloatSchema()},
]

field_dict = [
    # INT
    {
        "obj": {"type": "integer"},
        "class": faraday_integer.FaradayIntegerSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {"data": 1, "value": 1},
                    {"data": 1.5, "value": 1},
                    {"data": "1", "value": 1},
                ],
            },
            "ser": {
                "fields": [
                    {"data": 1, "value": 1},
                    {"data": 1.5, "value": 1},
                    {"data": "1", "value": 1},
                ],
            },
        },
        "invalid": ["test", {"data": "text"}, ["test", "test2"]],
    },
    # STRING
    {
        "obj": {"type": "string"},
        "class": faraday_string.FaradayStringSchema(),
        "valid": {
            "deser": {
                "fields": [{"data": "text_string", "value": "text_string"}],
            },
            "ser": {
                "fields": [{"data": "text_string", "value": "text_string"}],
            },
        },
        "invalid": [{"Test": 2}, ["test"]],
    },
    # BOOL
    {
        "obj": {"type": "boolean"},
        "class": faraday_boolean.FaradayBooleanSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {"data": True, "value": True},
                    {"data": "true", "value": True},
                    {"data": 1, "value": True},
                ],
            },
            "ser": {
                "fields": [
                    {"data": True, "value": True},
                    {"data": "true", "value": True},
                    {"data": 1, "value": True},
                ],
            },
        },
        "invalid": [
            "test",
            {"data": "test"},
            {"data": 3},
            3,
            ["3"],
        ],
    },
    # LIST
    {
        "obj": {"type": "list", "composed": (int, str)},
        "class": faraday_list.FaradayListSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {
                        "data": [1, "test_data"],
                        "value": [1, "test_data"],
                    }
                ],
            },
            "ser": {
                "fields": [
                    {
                        "data": [1, "test_data"],
                        "value": [1, "test_data"],
                    }
                ],
            },
        },
        "invalid": [
            "test",
            {"data": 1},
            {"data": "test"},
            {"data": {"test": "test"}},
            1,
            {"test": "test"},
        ],
    },
    # RANGE
    {
        "obj": {"type": "range"},
        "class": faraday_int_range.FaradayRangeSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {"data": "1-4", "value": [1, 2, 3, 4]},
                    {"data": [4, 5, 6, 7], "value": [4, 5, 6, 7]},
                ],
            },
            "ser": {
                "fields": [{"data": [1, 2, 3, 4], "value": "1-4"}],
            },
        },
        "invalid": [
            "test",
            {"data": "6-4"},
            {"data": "test"},
            {"data": 1},
            {"data": 1 - 4},
            {"data": [1, 2, "Test", 4]},
            "6-4",
            1,
            1 - 4,
            [1, 2, "Test", 4],
        ],
    },
    # IP
    {
        "obj": {"type": "ip"},
        "class": faraday_ip.FaradayIPSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {
                        "data": "192.168.0.1",
                        "value": IPv4Address("192.168.0.1"),
                    },
                    {
                        "data": "2001:db8:0:0:0:0:2:1",
                        "value": IPv6Address("2001:db8:0:0:0:0:2:1"),
                    },
                    {
                        "data": "2001:db8::2:1",
                        "value": IPv6Address("2001:db8:0:0:0:0:2:1"),
                    },
                ],
            },
            "ser": {
                "fields": [
                    {
                        "data": IPv4Address("192.168.0.1"),
                        "value": "192.168.0.1",
                    },
                    {
                        "data": IPv6Address("2001:db8:0:0:0:0:2:1"),
                        "value": "2001:db8::2:1",
                    },
                ],
            },
        },
        "invalid": [
            "test",
            {"data": "192.168..1"},
            {"data": "test"},
            {"data": [192, 168, 0, 1]},
            "192.168..1",
            [192, 168, 0, 1],
        ],
    },
    # Float
    {
        "obj": {"type": "float"},
        "class": faraday_float.FaradayFloatSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {"data": 1.5, "value": 1.5},
                    {"data": "1.5", "value": 1.5},
                    {"data": "1", "value": 1.0},
                ],
            },
            "ser": {
                "fields": [
                    {"data": 1.5, "value": 1.5},
                    {"data": "1.5", "value": 1.5},
                    {"data": "1", "value": 1.0},
                ],
            },
        },
        "invalid": ["test", {"data": "text"}],
    },
    # URL
    {
        "obj": {"type": "url"},
        "class": faraday_url.FaradayUrlSchema(),
        "valid": {
            "deser": {
                "fields": [
                    {"data": "http://test.com", "value": "http://test.com"},
                    {"data": "ftp://test.com.ar", "value": "ftp://test.com.ar"},
                    {"data": "https://mail.test.eu/test.php", "value": "https://mail.test.eu/test.php"},
                ],
            },
            "ser": {
                "fields": [
                    {"data": "http://test.com", "value": "http://test.com"},
                    {"data": "ftp://test.com.ar", "value": "ftp://test.com.ar"},
                    {"data": "https://mail.test.eu/test.php", "value": "https://mail.test.eu/test.php"},
                ],
            },
        },
        "invalid": [
            "www.google.com",
            "google.com",
            "localhost",
            "192.168.0.1",
            "test://www.google.com",
        ],
    },
]


@pytest.mark.parametrize("case", indentify_dict)
def test_indentify_type(case):
    # In base of obj, it should recognize the class
    assert isinstance(case["class"], type(DATA_TYPE[case["obj"]["type"]]))


@pytest.mark.parametrize("case", indentify_dict)
def test_to_object(case):
    assert case["class"].to_obj() == case["obj"]["type"]


# Here goes an import, or an example list
@pytest.mark.parametrize("field", field_dict)
def test_deserialize(field):
    fields = field["valid"]["deser"]["fields"]
    if field["obj"]["type"] == "list":
        field["class"]._composed_list = field["obj"]["composed"]
    for entry in fields:
        value = entry["value"]
        data = entry["data"]
        assert deserialize_param(field["class"], data) == value


@pytest.mark.parametrize("field", field_dict)
def test_serialize(field):
    fields = field["valid"]["ser"]["fields"]
    if field["obj"]["type"] == "list":
        field["class"]._composed_list = field["obj"]["composed"]
    for entry in fields:
        value = entry["value"]
        data = entry["data"]
        assert serialize_param(field["class"], data) == value


@pytest.mark.parametrize("field", field_dict)
def test_invalid_data(field):
    fields = field["invalid"]
    if field["obj"]["type"] == "list":
        field["class"]._composed_list = field["obj"]["composed"]
    for entry in fields:
        assert type_validate(field["class"], entry)


def test_OR_valid():
    # Range is the only valid type here
    _type = ["integer", "ip", "range", "float"]
    _data = "2-7"
    _expected = [2, 3, 4, 5, 6, 7]
    assert deserialize_param(_type, _data) == _expected


def test_OR_invalid():
    _type = ["integer", "ip", "range", "float"]
    _data = ["test"]
    errors = type_validate(_type, _data)
    assert errors


@pytest.mark.parametrize("type", ["test", "ipp", Schema()])
def test_invalid_data_type(type):
    with pytest.raises(ValidationError, match="Invalid Data Type"):
        type_validate(type, "test")



#!/usr/bin/env python

"""The setup script."""
from re import search
from setuptools import setup, find_packages

with open("faraday_agent_parameters_types/__init__.py", "rt", encoding="utf8") as f:
    version = search(r"__version__ = \"(.*?)\"", f.read()).group(1)


with open("README.md") as readme_file:
    readme = readme_file.read()

with open("CHANGELOG.md") as history_file:
    history = history_file.read()

requirements = [
    "marshmallow>=3.11.0",
    "packaging",
]

setup_requirements = [
    "pytest-runner",
]

extra_req = {
    "dev": ["giteasychangelog", "flake8", "pre-commit", "black"],
    "test": [
        "pytest",
        "pytest-cov",
    ],
    "docs": [
        "mkdocs",
        "mkdocs-material",
    ],
}

setup(
    author="Faraday Development Team",
    author_email="devel@infobytesec.com",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    description="""The faraday agents run code remotely to ensure your domains. This info is triggered and published
    to a faraday server instance, which had set the parameters of the code. This repository sets the models to be used
    by both sides.""",
    install_requires=requirements,
    extras_require=extra_req,
    license="GNU General Public License v3",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="faraday",
    name="faraday_agent_parameters_types",
    packages=find_packages(),
    setup_requires=setup_requirements,
    url="https://github.com/infobyte/faraday_agent_parameters_types",
    version=version,
    zip_safe=False,
)



from ..faraday_agent_parameters_types import TypeSchema
from marshmallow import fields
from dataclasses import dataclass, field


@dataclass
class FaradayInteger:
    data: int = int()
    class_name: str = field(default="integer", init=False)
    base: str = field(default="integer", init=False)


class FaradayIntegerSchema(TypeSchema):
    data = fields.Integer()
    type = FaradayInteger
